#!/usr/bin/perl 
#  makelib.pl
#
#  Compose a ordered library of object modules
#  The order of the object modules is defined by the .spec file.
#  In this file the object modules are ordered in such a way that
#  when an object module (1) needs another object module (2) (cq. has an 
#  external to that other module), object module (2) is after object
#  module (1) in the library. 
#  The .spec file for a library is generated by makelibspec.pl
#
#  Author: Theo Engel
#  Info:   Info@theoengel.nl
#
## Usage: makelib.pl [-sd=<sourcedir>] [-td=<targetdir>] <libname>
##        output: <libname>.lib
##        uses the file <libname>.lib.def to compose the library
#
#  The .def file is assumed in the <sourcedir>
#  The object modules for the library are asumed in the <targetdir>
#  The resulting library will also be stored in the <targetdir>
#  When either the <sourcedir> or <targetdir> are not specified,
#  the current directory is assumed.

use strict;
use bytes;
my $DEBUG = 0;

my @t;
my $sd=""; # source directory
my $td=""; # target directory
my $sdir="current";
my $tdir="current";
my $fi;

# argument 1
my $arg = shift (@ARGV);
if ($arg !~ /\-/ && $arg ne "")
{
	$fi=$arg;
}
elsif ($arg =~ /\-/)
{
	@t=split(/\=/,$arg);
	if($t[0] eq "-sd")
	{
		$sd=$t[1];
		$sdir=$sd;
	}
	elsif($t[0] eq "-td")
	{
		$td=$t[1];
		$tdir=$td;
	}
	else
	{
		print "Wrong 1st argument:$arg\n";
		print "Usage: makelib.pl [-sd=<sourcedir>] [-td=targetdir] <libname>\n";
		exit 1;
	}
	# argument 2
	$arg = shift (@ARGV);
	if ($arg !~ /\-/ && $arg ne "")
	{
		$fi = $arg;
	}
	elsif ($arg =~ /\-/)
	{
		@t=split(/\=/,$arg);
		if($t[0] eq "-sd")
		{
			$sd=$t[1];
			$sdir=$sd;			
		}
		elsif($t[0] eq "-td")
		{
			$td=$t[1];
			$tdir=$td;			
		}
		else
		{
			print "Wrong 2nd argument:$arg\n";
		    print "Usage: makelib.pl [-sd=<sourcedir>] [-td=targetdir] <libname>\n";			
			exit 1;
		}
		$arg = shift (@ARGV);
		if ($arg !~ /\-/ && $arg ne "")
		{
			$fi = $arg;
		}
		else
		{
			print "Wrong 3rd argument:$arg\n";
		    print "Usage: makelib.pl [-sd=<sourcedir>] [-td=targetdir] <libname>\n";			
			exit 1;
		}
	}
}
else
{
	print "No or wrong arguments\n";
	print "Usage: makelib.pl [-sd=<sourcedir>] [-td=targetdir] <libname>\n";	
	exit 1;
}

my $flib  = $td . '/' . $fi . '.lib';			# library file (output)
my $fspec = $sd . '/' . $fi . '.lib.spec';      # lib spec file
my $fobj;                                       # module object file

my $exec = "./makelibspec.pl -sd=$sd -td=$td $fi\n";
if(system($exec))
{
	print "Error while making $fspec\n";
	exit 1;
}

print "Make object library $flib according $fspec\n";
print "Start reading $fspec\n";

open (IN,$fspec) || die "cannot open specfile $fspec: $!";
open (BIN, ">:raw", $flib) || die "cannot open outputfile $flib: $!";

my $cin=0;		# number of module names read in from spec file
my $leader="false";
my $c;			# char input record
my $end="";
my $blty;

my $eom     = 0203;
my $null 	= 0;
my $som     = 0201;

while (<IN>)
{ 
	#print "$_";
	$c=substr($_,0,1);
	#print "c:$c\n";
	next if($c eq '#' || $c eq '*'); # skip comment lines
	if($_ =~ ':EOF')
	{
		last;
	}
	$cin=$cin+1;
	chomp($_);
	$fobj= $td . '/' . $_;
	print "Get object module: $fobj\n";
	open (OBJ, $fobj) || die "cannot open objectfile $fobj: $!";
	
    # copy .obj => .lib
	$end="false";
	$leader="false";
	while( 1 )
	{
		nextchar();
		#skip leader in case there
		if ($leader eq "false")
		{
			next if $c eq $null;
			$leader = "true";
			#print "Leader skipped\n";
		}
		if($c == 0xFF)
		{
			# start of object module
			punch($c);
			while(1)
			{
				nextchar();
				punch($c);				
				if($c == $som)
				{
					# punch block
					$blty = nextchar();
					punch($c);
					if($blty == 7)
					{
						$end="true";
					}
					while(1)
					{
						nextchar();
						punch($c);						
						if($c == $eom)
						{
							last;
						}
					}
				}
				# end block ?
				if($end eq "true")
				{
					last;
				}
			}
			punch($null);
			punch($null);
			punch($null);
			punch($null);
			last;
		}
	}
	close (OBJ) || die "can't close $fobj: $!";
}

close (IN)  || die "can't close $fspec: $!";
close (BIN) || die "can't close $flib: $!";

print "End of library composition\n";
print "$cin  modules copied to $flib\n";


sub nextchar {
	    if(read(OBJ,$c,1) == 0)
		{
			return -1;						#EOF
		}
		$c = unpack("C", $c); 	            # get new current char
		return ($c & 0xFF);
}

sub punch {
    my($a) = @_;
    my $byte=0;
    $byte = pack("C", $a);
    print BIN $byte;
}

